@@ -43,7 +43,7 @@ advfq_dtdy <- derive_vars_merged(
   ungroup(qs),
   dataset_add = adsl,
   new_vars = adsl_vars,
-  by_vars = exprs(STUDYID, USUBJID)
+  by_vars = get_admiral_option("subject_keys")
 ) %>%
   ## Calculate ADT, ADY ----
   derive_vars_dt(
@@ -75,11 +75,16 @@ advfq_aval <- advfq_dtdy %>%
 # else set to 0 if [advfq.AVAL] = 5
 advfq_qr01 <- advfq_aval %>%
   derive_summary_records(
-    by_vars = exprs(STUDYID, USUBJID, !!!adsl_vars, PARAMCD, VISITNUM, VISIT, ADT, ADY),
-    filter = QSTESTCD == "VFQ1" & !is.na(AVAL),
-    analysis_var = AVAL,
-    summary_fun = identity,
-    set_values_to = exprs(PARAMCD = "QR01")
+    dataset_add = advfq_aval,
+    by_vars = c(
+      get_admiral_option("subject_keys"),
+      exprs(!!!adsl_vars, PARAMCD, VISITNUM, VISIT, ADT, ADY)
+    ),
+    filter_add = QSTESTCD == "VFQ1" & !is.na(AVAL),
+    set_values_to = exprs(
+      AVAL = identity(AVAL),
+      PARAMCD = "QR01"
+    )
   ) %>%
   mutate(AVAL = ifelse(PARAMCD == "QR01",
     case_when(
@@ -101,11 +106,16 @@ advfq_qr01 <- advfq_aval %>%
 # else set to 0 if [advfq.AVAL] = 6
 advfq_qr02 <- advfq_qr01 %>%
   derive_summary_records(
-    by_vars = exprs(STUDYID, USUBJID, !!!adsl_vars, PARAMCD, VISITNUM, VISIT, ADT, ADY),
-    filter = QSTESTCD == "VFQ2" & !is.na(AVAL),
-    analysis_var = AVAL,
-    summary_fun = identity,
-    set_values_to = exprs(PARAMCD = "QR02")
+    dataset_add = advfq_qr01,
+    by_vars = c(
+      get_admiral_option("subject_keys"),
+      exprs(!!!adsl_vars, PARAMCD, VISITNUM, VISIT, ADT, ADY)
+    ),
+    filter_add = QSTESTCD == "VFQ2" & !is.na(AVAL),
+    set_values_to = exprs(
+      AVAL = identity(AVAL),
+      PARAMCD = "QR02"
+    )
   ) %>%
   mutate(AVAL = ifelse(PARAMCD == "QR02",
     case_when(
@@ -127,11 +137,16 @@ advfq_qr02 <- advfq_qr01 %>%
 # else set to 0 if [advfq.AVAL] = 1
 advfq_qr03 <- advfq_qr02 %>%
   derive_summary_records(
-    by_vars = exprs(STUDYID, USUBJID, PARAMCD, !!!adsl_vars, VISITNUM, VISIT, ADT, ADY),
-    filter = QSTESTCD == "VFQ3" & !is.na(AVAL),
-    analysis_var = AVAL,
-    summary_fun = identity,
-    set_values_to = exprs(PARAMCD = "QR03")
+    dataset_add = advfq_qr02,
+    by_vars = c(
+      get_admiral_option("subject_keys"),
+      exprs(PARAMCD, !!!adsl_vars, VISITNUM, VISIT, ADT, ADY)
+    ),
+    filter_add = QSTESTCD == "VFQ3" & !is.na(AVAL),
+    set_values_to = exprs(
+      AVAL = identity(AVAL),
+      PARAMCD = "QR03"
+    )
   ) %>%
   mutate(AVAL = ifelse(PARAMCD == "QR03",
     case_when(
@@ -153,11 +168,16 @@ advfq_qr03 <- advfq_qr02 %>%
 # else set to 0 if [advfq.AVAL] = 1
 advfq_qr04 <- advfq_qr03 %>%
   derive_summary_records(
-    by_vars = exprs(STUDYID, USUBJID, PARAMCD, !!!adsl_vars, VISITNUM, VISIT, ADT, ADY),
-    filter = QSTESTCD == "VFQ4" & !is.na(AVAL),
-    analysis_var = AVAL,
-    summary_fun = identity,
-    set_values_to = exprs(PARAMCD = "QR04")
+    dataset_add = advfq_qr03,
+    by_vars = c(
+      get_admiral_option("subject_keys"),
+      exprs(PARAMCD, !!!adsl_vars, VISITNUM, VISIT, ADT, ADY)
+    ),
+    filter_add = QSTESTCD == "VFQ4" & !is.na(AVAL),
+    set_values_to = exprs(
+      AVAL = identity(AVAL),
+      PARAMCD = "QR04"
+    )
   ) %>%
   mutate(AVAL = ifelse(PARAMCD == "QR04",
     case_when(
@@ -175,11 +195,16 @@ advfq_qr04 <- advfq_qr03 %>%
 # Average of QR01 and QR02 records
 advfq_qsg01 <- advfq_qr04 %>%
   derive_summary_records(
-    by_vars = exprs(STUDYID, USUBJID, !!!adsl_vars, VISITNUM, VISIT, ADT, ADY),
-    filter = PARAMCD %in% c("QR01", "QR02") & !is.na(AVAL),
-    analysis_var = AVAL,
-    summary_fun = mean,
-    set_values_to = exprs(PARAMCD = "QSG01")
+    dataset_add = advfq_qr04,
+    by_vars = c(
+      get_admiral_option("subject_keys"),
+      exprs(!!!adsl_vars, VISITNUM, VISIT, ADT, ADY)
+    ),
+    filter_add = PARAMCD %in% c("QR01", "QR02") & !is.na(AVAL),
+    set_values_to = exprs(
+      AVAL = mean(AVAL),
+      PARAMCD = "QSG01"
+    )
   )
 
 ## Derive a new record as a summary record  ----
@@ -187,11 +212,16 @@ advfq_qsg01 <- advfq_qr04 %>%
 # Average of QR03 and QR04 records
 advfq_qsg02 <- advfq_qsg01 %>%
   derive_summary_records(
-    by_vars = exprs(STUDYID, USUBJID, !!!adsl_vars, VISITNUM, VISIT, ADT, ADY),
-    filter = PARAMCD %in% c("QR03", "QR04") & !is.na(AVAL),
-    analysis_var = AVAL,
-    summary_fun = mean,
-    set_values_to = exprs(PARAMCD = "QSG02")
+    dataset_add = advfq_qsg01,
+    by_vars = c(
+      get_admiral_option("subject_keys"),
+      exprs(!!!adsl_vars, VISITNUM, VISIT, ADT, ADY)
+    ),
+    filter_add = PARAMCD %in% c("QR03", "QR04") & !is.na(AVAL),
+    set_values_to = exprs(
+      AVAL = mean(AVAL),
+      PARAMCD = "QSG02"
+    )
   )
 
 
@@ -200,11 +230,16 @@ advfq_qsg02 <- advfq_qsg01 %>%
 # Average of QSG01 and QSG02 records
 advfq_qbcs <- advfq_qsg02 %>%
   derive_summary_records(
-    by_vars = exprs(STUDYID, USUBJID, !!!adsl_vars, VISITNUM, VISIT, ADT, ADY),
-    filter = PARAMCD %in% c("QSG01", "QSG02") & !is.na(AVAL),
-    analysis_var = AVAL,
-    summary_fun = sum,
-    set_values_to = exprs(PARAMCD = "QBCSCORE")
+    dataset_add = advfq_qsg02,
+    by_vars = c(
+      get_admiral_option("subject_keys"),
+      exprs(!!!adsl_vars, VISITNUM, VISIT, ADT, ADY)
+    ),
+    filter_add = PARAMCD %in% c("QSG01", "QSG02") & !is.na(AVAL),
+    set_values_to = exprs(
+      AVAL = sum(AVAL),
+      PARAMCD = "QBCSCORE"
+    )
   )
 
 ## Get visit info ----
@@ -220,11 +255,14 @@ advfq_visit <- advfq_qbcs %>%
       !is.na(VISIT) ~ str_to_title(VISIT),
       TRUE ~ NA_character_
     ),
-    AVISITN = as.numeric(case_when(
+    AVISITN = case_when(
       VISIT == "BASELINE" ~ "0",
-      str_detect(VISIT, "WEEK") ~ str_trim(str_replace(VISIT, "WEEK", "")),
-      TRUE ~ NA_character_
-    ))
+      str_detect(VISIT, "WEEK") ~ VISIT %>%
+        str_replace("WEEK", "") %>%
+        str_replace("\\(T\\)", "") %>%
+        str_trim(),
+      TRUE ~ NA
+    ),
   )
 
 advfq_ontrt <- advfq_visit %>%
@@ -242,20 +280,19 @@ advfq_blfl <- advfq_ontrt %>%
   restrict_derivation(
     derivation = derive_var_extreme_flag,
     args = params(
-      by_vars = exprs(STUDYID, USUBJID, PARAMCD),
+      by_vars = c(get_admiral_option("subject_keys"), exprs(PARAMCD)),
       order = exprs(ADT, VISITNUM, QSSEQ),
       new_var = ABLFL,
       mode = "last"
     ),
-    filter = (!is.na(AVAL) &
-      ADT <= TRTSDT)
+    filter = (!is.na(AVAL) & ADT <= TRTSDT)
   )
 
 ## Derive baseline information ----
 advfq_change <- advfq_blfl %>%
   # Calculate BASE
   derive_var_base(
-    by_vars = exprs(STUDYID, USUBJID, PARAMCD),
+    by_vars = c(get_admiral_option("subject_keys"), exprs(PARAMCD)),
     source_var = AVAL,
     new_var = BASE
   ) %>%
@@ -271,7 +308,7 @@ advfq_anlflag <- advfq_change %>%
     derivation = derive_var_extreme_flag,
     args = params(
       new_var = ANL01FL,
-      by_vars = exprs(USUBJID, PARAMCD, AVISIT),
+      by_vars = c(get_admiral_option("subject_keys"), exprs(PARAMCD, AVISIT)),
       order = exprs(ADT, AVAL),
       mode = "last"
     ),
@@ -283,7 +320,7 @@ advfq_aseq <- advfq_anlflag %>%
   # Calculate ASEQ
   derive_var_obs_number(
     new_var = ASEQ,
-    by_vars = exprs(STUDYID, USUBJID),
+    by_vars = get_admiral_option("subject_keys"),
     order = exprs(PARAMCD, ADT, AVISITN, VISITNUM),
     check_type = "error"
   ) %>%
@@ -295,7 +332,7 @@ advfq_aseq <- advfq_anlflag %>%
 advfq_adsl <- advfq_aseq %>%
   derive_vars_merged(
     dataset_add = select(adsl, !!!negate_vars(adsl_vars)),
-    by_vars = exprs(STUDYID, USUBJID)
+    by_vars = get_admiral_option("subject_keys")
   )
 
 # Final Steps, Select final variables and Add labels
@@ -308,10 +345,10 @@ admiralophtha_advfq <- advfq_adsl
 
 # Save output ----
 
-dir <- file.path(getwd(), "tmp")
-print(dir)
+dir <- tools::R_user_dir("admiralophtha_templates_data", which = "cache")
+# Change to whichever directory you want to save the dataset in
 if (!file.exists(dir)) {
   # Create the folder
-  dir.create(dir)
+  dir.create(dir, recursive = TRUE, showWarnings = FALSE)
 }
 save(admiralophtha_advfq, file = file.path(dir, "advfq.rda"), compress = "bzip2")
